package es.unican.docencia.y2011.eda.listaEnlazada.modular.Generica;

/**
 * Clase lista enlazada (versin no orientada a objetos y generica).
 * @author Pablo Sanchez (p.sanchez@unican.es)
 * 		   http://personales.unican.es/sanchezbp	
 */
public class ListaEnlazada<Elemento> {
	
	// Definicin de la tupla ListaEnlazadaNaturales 
	protected Elemento dato;
	protected ListaEnlazada<Elemento> sig;
		
	// Implementamos ahora las operaciones requeridas de forma esttica
	
	//-------------------------------------------------------------------//
	// Operaciones publicas sobre la clase								 //  
	//-------------------------------------------------------------------//
	
	/**
	 * Metodo que devuelve una lista vacia 
	 * @return Una lista vacia
	 */
	static public <T> ListaEnlazada<T> listaVacia() {
		return null;
	} // 
	
	/**
	 * Devuelve verdadero si la lista esta vacia, es decir, sino contiene 
	 * ningun elemento. Devuelve falso en caso contrario
	 * @param l Una lista enlazada 
	 * @return Verdadero si la lista esta vacia. Falso en caso contrario
	 */
	// FUNCION esVacia(l : ListaEnlazadaChinos) : Booleano;
	static public boolean esVacia(ListaEnlazada<?> l) {
		return (l == null);
	} // esVacia
	
	/**
	 * Inserta un elemento en la lista enlazada. 
	 * NOTA: No se comprueba que est repetido 
	 * @param l La lista enlazada donde se inserta el elemento
	 * @return La lista enlazada l con el elemento insertado 
	 */
	// PROCEDIMIENTO inserta(REF l : ListaEnlazadaChinos, c : Chinos)
	// NOTA: Observad como el paso por referencia es simulado mediante la 
	//       devolucin de la lista posiblemente modificada  
	public static <T> ListaEnlazada<T> inserta(ListaEnlazada<T> l, T c) {
		ListaEnlazada<T> aux = new ListaEnlazada<T>();
		aux.dato = c;
		aux.sig   = l;
		l = aux;
		// OJO: Si yo no recojo lo que me devuelve el metodo, la lista no se modifica,
		//      puesto que esto no es paso por referencia
		return l;
	} // inserta
	
	/**
	 * Borra un elemento de nuestra lista, concretamente el primero que se encuentra con el nombre
	 * dado. Si no se encuentra el elemento, no se hace nada 
	 * @param l La lista enlazada donde debemos borrar al elemento  
	 * @param clave La clave del elemento a borrar
	 * @return La lista enlazada l con el elemento borrado
	 */
	// PROC borra(REF l : ListaEnlazadaAlumnos, clave : Clave)
	public static <T extends IClave<C>, C> ListaEnlazada<T> borra(ListaEnlazada<T> l, C clave) {
		// Si est vaca, la dejamos vaca
		if (l != null) {
			// Obviamente {P:(l != null)} se satisface en este punto y puedo dereferenciar el 
			// puntero sin problemas
			if (l.dato.getClave().equals(clave)) {
				// Tengo que borrar el primer elemento

				//-----------------------------------------------------------------------
				// Esta lnea en Java no es necesaria, porque Java, para bien o para mal
				// recoge nuestra basura
				//-----------------------------------------------------------------------
				// ListaEnlazadaChinos aux = l; 
				
				// Avanzamos un elemento. Nadie apunta ahora la primer elemento de la lista, 
				// por lo que el recolector automtico de basura (garbage collector) algn da
				// liberar ese espacio de memoria
				l = l.sig;
				
				//-------------------------------------------------------------------------------
				// Si no hubiese recolector automtico de basura tendramos que aadir una linea
				// como la de abajo
				//-------------------------------------------------------------------------------
				// free(aux);
			} else {
				// Itero mientras me queden elementos por analizar y no haya encontrado al chino
				// Itero desde una posicin ms atrasada de la que analizo
				ListaEnlazada<T> aux = l; 
				while ((aux.sig != null) && (!aux.sig.dato.getClave().equals(clave))) {
					aux = aux.sig;
				} // while
				// En este punto o lo he encontrado o he llegado al final de la lista
				// Dicho predicado se asegura mediante la negacin de la condicin del bucle
				// Si lo he encontrado he de borrarlo, sino no hago nada
				if (aux.sig != null) {
					// Simplemente avanzo el puntero
					// Si Java no tuviese recoleccion automatica de basura, tendra que hacer lo mismo
					// que para el caso anterior, si deseo liberar el espacio en memoria ocupado por 
					// el nodo.
					aux.sig = aux.sig.sig;
				} // if
			} // if
		} // if
		
		return l;
	} // borra
	
	/**
	 * Devuelve el primer elemento almacenado en la lista cuya clave coincide con la solicitada 
	 * @param l La lista enlazada donde se busca el elemento
	 * @param clave La clave del elemento que queremos recuperar
	 * @return Un puntero al primer elemento cuyo nombre coincide con el indicado, nulo si tal elemento
	 *         no se encuentra en nuestra lista
	 */
	// FUN busca(l : ListaEnlazadaAlumnos, n : Clave) : PUNTERO A Chino
	public static <T extends IClave<Clave>, Clave> T busca(ListaEnlazada<T> l, Clave clave) {
		// Simplemente itero por la lista
		T result = null;
		l = buscaNodo(l, clave);
		// Si lo he encontrado, lo devuelvo 
		if (l != null) {
			result = l.dato; 
		} // if
		return result;
	} //  busca

	/**
	 * Cambia el primer elemento cuya clave es igual a la que se pasa como parametro por 
	 * nuevoElemento
	 * @param l Lista sobre la que se actualiza 
	 * @param clave Clave del elemento a sustituir
	 * @param nuevoDato Elemento que reemplaza al elemento a sustituir
	 * @return La lista actualizada 
	 */
	// PROC actualiza(l : ListaEnlazada, clave : Clave, nuevo : Chino);
	public static <T extends IClave<C>, C> ListaEnlazada<T> actualiza(ListaEnlazada<T> l, C clave, T nuevoDato) {
		
		ListaEnlazada<T> nodo = buscaNodo(l,clave);
		if (nodo != null) {
			nodo.dato = nuevoDato;
		} // if
		
		return l;
	} // actualiza
		
	//-----------------------------------------------------------------------------------//
	// 						Funciones auxiliares (helper functions)						 //
	//-----------------------------------------------------------------------------------//	
	
	/**
	 * Devuelve una referencia al nodo que contiene al primer elemento cuya clave se pasa como 
	 * parametro  
	 * @param l Lista donde se busca al elemento
	 * @param clave Clave del elemento que buscamos
	 * @return Referencia al nodo que contiene al primer elemento buscado si este existe, nulo 
	 * 		   en caso contrario	 
	 */
	protected static <T extends IClave<C>, C> ListaEnlazada<T> buscaNodo(ListaEnlazada<T> l,  C clave) {
		
		ListaEnlazada<T> result = null;
		
		while ((l != null) && (!l.dato.getClave().equals(clave))) {
			l = l.sig;
		} // while
		
		if (l != null) {
			result = l;
		} // if
		
		return result;
	} // buscaNodo
	
	//-----------------------------------------------------------------------------------//
	// 						Aburrida y soporfera entrada/salida						 //
	//-----------------------------------------------------------------------------------//	
	
	/**
	 * Devuelve una cadena con los nombre de todos los elementos seriliazados por orden de insercion
	 * inverso, es decir, el ultimo elemento insertado es el primero que aparece
	 * @return La mencionada cadena. 
	 */
	public static String toString(ListaEnlazada<?> l) {
		
		String result = "";
		
		while(l != null) {
			result = result + l.dato.toString() + "; ";
			l = l.sig;
		} //  while
		
		return result;
	} // 
	
} // ListaEnlazada<Elemento>
