package es.unican.docencia.y2011.eda.listaEnlazada.modular.noGenerica;

/**
 * Clase lista enlazada (versin no orientada a objetos).
 * ADVERTENCIA: La practica esta hecha sin usar genericos, 
 * porque el sistema de genericos de Java no es muy compatible 
 * con los metodos estaticos. 
 * @author Pablo Sanchez (p.sanchez@unican.es)
 * 		   http://personales.unican.es/sanchezbp	
 */
public class ListaEnlazadaChinos {
	
	// Definicin de la tupla ListaEnlazadaNaturales 
	protected Chino chino;
	protected ListaEnlazadaChinos sig;
		
	// Implementamos ahora las operaciones requeridas de forma esttica
	
	//-------------------------------------------------------------------//
	// Operaciones publicas sobre la clase								 //  
	//-------------------------------------------------------------------//
	
	/**
	 * Metodo que devuelve una lista vacia 
	 * @return Una lista vacia
	 */
	static public ListaEnlazadaChinos listaVacia() {
		return null;
	} // 
	
	/**
	 * Devuelve verdadero si la lista esta vacia, es decir, sino contiene 
	 * ningun elemento. Devuelve falso en caso contrario
	 * @param l Una lista enlazada de naturales
	 * @return verdadero si la lista esta vacia. Falso en caso contrario
	 */
	// FUNCION esVacia(l : ListaEnlazadaChinos) : Booleano;
	static public boolean esVacia(ListaEnlazadaChinos l) {
		return (l == null);
	} // esVacia
	
	/**
	 * Inserta un elemento en la lista enlazada. 
	 * NOTA: No se comprueba que est repetido 
	 * @param l La lista enlazada donde se inserta el elemento
	 * @return La lista enlazada l con el elemento insertado 
	 */
	// PROCEDIMIENTO inserta(REF l : ListaEnlazadaChinos, c : Chinos)
	// NOTA: Observad como el paso por referencia es simulado mediante la 
	//       devolucin de la lista posiblemente modificada  
	public static ListaEnlazadaChinos inserta(ListaEnlazadaChinos l, Chino c) {
		if (l == null) {
			l = new ListaEnlazadaChinos();
			l.chino = c;
			l.sig   = null;
		} else {
			ListaEnlazadaChinos aux = new ListaEnlazadaChinos();
			aux.chino = c;
			aux.sig   = l;
			l = aux;
		}// if
		// OJO: Si yo no recojo lo que me devuelve el metodo, la lista no se modifica,
		//      puesto que esto no es paso por referencia
		return l;
	} // inserta
	
	/**
	 * Borra un chino de nuestra lista, concretamente el primero que se encuentra con el nombre
	 * dado. Si no se encuentra el elemento, no se hace nada 
	 * @param l La lista enlazada donde debemos borrar al chino  
	 * @param name El nombre del chino a borrar
	 * @return La lista enlazada l con el elemento borrado
	 */
	// PROC borra(REF l : ListaEnlazadaAlumnos, clave : Clave)
	public static ListaEnlazadaChinos borra(ListaEnlazadaChinos l, String name) {
		// Si est vaca, la dejamos vaca
		if (l != null) {
			// Obviamente {P:(l != null)} se satisface en este punto y puedo dereferenciar el 
			// puntero sin problemas
			if (l.chino.name.equals(name)) {
				// Tengo que borrar el primer elemento

				//-----------------------------------------------------------------------
				// Esta lnea en Java no es necesaria, porque Java, para bien o para mal
				// recoge nuestra basura
				//-----------------------------------------------------------------------
				// ListaEnlazadaChinos aux = l; 
				
				// Avanzamos un elemento. Nadie apunta ahora la primer elemento de la lista, 
				// por lo que el recolector automtico de basura (garbage collector) algn da
				// liberar ese espacio de memoria
				l = l.sig;
				
				//-------------------------------------------------------------------------------
				// Si no hubiese recolector automtico de basura tendramos que aadir una linea
				// como la de abajo
				//-------------------------------------------------------------------------------
				// free(aux);
			} else {
				// Itero mientras me queden elementos por analizar y no haya encontrado al chino
				// Itero desde una posicin ms atrasada de la que analizo
				ListaEnlazadaChinos aux = l; 
				while ((aux.sig != null) && (!aux.sig.chino.name.equals(name))) {
					aux = aux.sig;
				} // while
				// En este punto o lo he encontrado o he llegado al final de la lista
				// Dicho predicado se asegura mediante la negacin de la condicin del bucle
				// Si lo he encontrado he de borrarlo, sino no hago nada
				if (aux.sig != null) {
					// Simplemente avanzo el puntero
					// Si Java no tuviese recoleccion automatica de basura, tendra que hacer lo mismo
					// que para el caso anterior, si deseo liberar el espacio en memoria ocupado por 
					// el nodo.
					aux.sig = aux.sig.sig;
				} // if
			} // if
		} // if
		
		return l;
	} // borra
	
	/**
	 * Devuelve el primer chino almacenado en la lista cuyo nombre coincide con el solicitado 
	 * @param l La lista enlazada donde se busca el elemento
	 * @param name El nombre del chino que queremos recuperar
	 * @return Un puntero al primer chino cuyo nombre coincide con el indicado, nulo si tal chino
	 *         no se encuentra en nuestra lista
	 */
	// FUN busca(l : ListaEnlazadaAlumnos, n : Clave) : PUNTERO A Chino
	public static Chino busca(ListaEnlazadaChinos l, String name) {
		// Simplemente itero por la lista
		Chino result = null;
		l = buscaNodo(l, name);
		// Si lo he encontrado, lo devuelvo 
		if (l != null) {
			result = l.chino; 
		} // if
		return result;
	} //  busca

	/**
	 * Cambia el primer chino cuyo nombre es igual al que se pasa como parametro al valor de nuevoChino 
	 * @param l Lista sobre la que se actualiza 
	 * @param name Nombre del chino a modificar
	 * @param nuevoChino Valores para el nueco chino
	 * @return La lista actualizada 
	 */
	// PROC actualiza(l : ListaEnlazada, clave : Clave, nuevo : Chino);
	public static ListaEnlazadaChinos actualiza(ListaEnlazadaChinos l, String name, Chino nuevoChino) {
		
		ListaEnlazadaChinos nodo = buscaNodo(l,name);
		if (nodo != null) {
			nodo.chino = nuevoChino;
		} // if
		
		return l;
	} // actualiza
		
	//-----------------------------------------------------------------------------------//
	// 						Funciones auxiliares (helper functions)						 //
	//-----------------------------------------------------------------------------------//	
	
	/**
	 * Devuelve una referencia al nodo que contiene al primer chino cuyo nombre se pasa como 
	 * parametro  
	 * @param l Lista donde se busca al elemento
	 * @param name Nombre del chino que buscamos
	 * @return Referencia al nodo que contiene al primer chino buscado si este existe, nulo 
	 * 		   en caso contrario	 
	 */
	protected static ListaEnlazadaChinos buscaNodo(ListaEnlazadaChinos l, String name) {
		
		ListaEnlazadaChinos result = null;
		
		while ((l != null) && (!l.chino.name.equals(name))) {
			l = l.sig;
		} // while
		
		if (l != null) {
			result = l;
		} // if
		
		return result;
	} // buscaNodo
	
	//-----------------------------------------------------------------------------------//
	// 						Aburrida y soporfera entrada/salida						 //
	//-----------------------------------------------------------------------------------//	
	
	/**
	 * Devuelve una cadena con los nombre de todos los chinos serliazados por orden de insercion
	 * inverso, es decir, el ultimo chino insertado es el primero que aparece
	 * @return La mencionada cadena. 
	 */
	public static String toString(ListaEnlazadaChinos l) {
		
		String result = "";
		
		while(l != null) {
			result = result + l.chino.getName() + "; ";
			l = l.sig;
		} //  while
		
		return result;
	} // 
	
} // ListaEnlazada<Elemento>
