package es.unican.docencia.y2011.eda.listaEnlazada.objetos.generico;

/**
 * Clase que representa a un chino. Es una clase pasiva que simplemente
 * almacena datos.
 * @author Pablo Sanchez (p.sanchez@unican.es)
 * 		   http://personales.unican.es/sanchezbp	
 */
public class Chino implements IClave<String> {
	
	// Nombre del chino
	protected String name = null;
	// Edad del chino
	// Invariante: (edad > 0)
	protected int edad = 0;
	
	/**
	 * Devuelve el nombre de un chino
	 * @return El nombre del chino
	 */
	public String getName() {
		return name;
	} // getName
	
	/**
	 * Asigna un nombre a un chino
	 * @param name El nombre a ser asignado al chino
	 */
	public void setName(String name) {
		this.name = name;
	} // setName
	
	/**
	 * Devuelve la edad actual del chino en cuestin 
	 * @return La edad del chino
	 * 	       Post: getEdad() > 0 
	 */
	public int getEdad() {
		return edad;
	} // getEdad 
	
	/**
	 * Asigna una determinada edad al chino actual
	 * @param edad La edad a ser asignada
	 * 		  Pre: edad > 0
	 */
	public void setEdad(int edad) {
		if (edad >= 0) {
			this.edad = edad;
		} else {
			// No se cumple la precondicion. En este caso aviso al 
			// que me ha llamado
			throw new IllegalArgumentException("El chino no puede tener edad negativa");
		} // if
	} // setEdad

	@Override
	/**
	 * Devuelve la clave para identificar chinos, que es simplemente su nombre
	 * @returns El nombre del chino
	 */
	public String getClave() {
		return this.getName();
	} // getClave
	
	/**
	 * Devuelve el nombre del chino como una cadena
	 * @return Cadena con el nombre del chino 
	 */
	public String toString() {
		return this.getName();
	} // toString
		
} // Chino
