package es.unican.docencia.y2011.eda.listaEnlazada.objetos.generico;

/**
 * Clase que sirve para lanzar el programa de prueba para el 
 * ejemplo de lista enlazada versin orientada a objetos 
 * @author Pablo Sanchez (p.sanchez@unican.es)
 * 		   http://personales.unican.es/sanchezbp	
 */
public class Runner {

	/**
	 * Metodo de entrada a la aplicacion
	 * @param args Argumentos de la linea de comandos. Son simplemente ignorados
	 */
	public static void main(String[] args) {
		
		// Creamos algunos chinos mas o menos famosos
		Chino mao = new Chino();
		mao.setName("Mao Tse Tung");
		mao.setEdad(117);
		
		Chino zhang = new Chino();
		zhang.setName("Zhang Yimou");
		zhang.setEdad(59);
		
		Chino liu = new Chino();
		liu.setName("Liu Xiaobo");
		liu.setEdad(54);
 
		ListaEnlazada<Chino,String> lista = ListaEnlazada.listaVacia();
		
		lista = lista.inserta(mao);
		lista = lista.inserta(zhang);
		lista = lista.inserta(liu);
		
		System.out.println("Lista de chinos = " + lista);
		
		// Acabamos con la lista
		lista = lista.borra("Mao Tse Tung");
		System.out.println("Lista de chinos (Borrado Mao)   = " + lista);
		lista = lista.borra("Zhang Yimou");
		System.out.println("Lista de chinos (Borrado Zhang) = " + lista);
		lista = lista.borra("Liu Xiaobo");
		System.out.println("Lista de chinos (Borrado Liu)   = " + lista);
		
		if (lista.esVacia()) {
			System.out.println("La lista est vaca");
		} //if

		// Recreamos la lista
		lista = lista.inserta(mao);
		lista = lista.inserta(zhang);
		lista = lista.inserta(liu);

		// Buscamos a Mao
		Chino mao2 = lista.busca("Mao Tse Tung");
		
		if (mao2 != null) {
			System.out.println("Chino encontrado con nombre " + mao2.getName() + " y edad " + mao2.getEdad());
		} // if
		
		// Este chino obviamente no existe 
		Chino noExiste = lista.busca("Alfredo Landa");
		if (noExiste == null) {
			System.out.println("No parece haber ningun chino llamado Alfredo Landa");
		} // if
		
		// Por problemas politicos, cambiamos a Liu por otro chino,
		Chino bruce = new Chino();
		bruce.setName("Bruce Lee");
		bruce.setEdad(70);
		
		lista = lista.actualiza("Liu Xiaobo", bruce);
		System.out.println("Lista de chinos (actualiza 1) = " + lista);
		
		// Vemos que pasa si queremos cambiar a Alfredo Landa (que no existe) por Mao
		lista = lista.actualiza("Alfredo Landa", mao);
		System.out.println("Lista de chinos (actualiza 2) = " + lista);
		
	} // main

} // Runner
